### BEGIN LICENSE
# Copyright (C) 2012 <Elio Esteves Duarte> <elio.esteves.duarte@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import locale
from locale import gettext as _

from gi.repository import GObject
from tomate.indicator import Indicator
from tomate.launcher import Launcher
from tomate.notification import OSDNotify, SoundNotify
from tomate.timer import Timer
from tomate.helpers import get_settings
from tomate.widgets.window import Window

locale.textdomain('tomate')


class Handler(object):
    """Main class of the application."""

    _POMODORO_OSD_TITLE = _('Pomodoro')
    _POMODORO_OSD_MSG = _("It's time to work. Focus now!")
    _SHORT_BREAK_OSD_TITLE = _('Short break')
    _SHORT_BREAK_OSD_MSG = _('Go take a coffee!')
    _LONG_BREAK_OSD_TITLE = _('Long break')
    _LONG_BREAK_OSD_MSG = _('Time to rest. Go take a walk!')
    _ALARM_OSD_TITLE = _('Alarm')
    _ALARM_OSD_MSG = _('The time is up!')

    def __init__(self):
        self.timer = Timer()
        self._osd = OSDNotify()
        self._sound = SoundNotify()
        self.indicator = Indicator(self)
        self.ui = Window(self)
        self._launcher = Launcher(self)
        self.settings = get_settings()

    def start_timer(self):
        """Start the timer and show the osd notification."""
        task = self.ui.selected_task
        minutes = self.settings.get_int(task + '-length')
        seconds = minutes * 60
        self.timer.start(seconds)
        self.indicator.set_status_active()

        if self.settings.get_boolean('osd-notification'):
            title, message = self._get_osd_messages(task)
            self._osd.display(title, message)

        GObject.timeout_add(1000, self.__update)

    def stop_timer(self):
        """Stop the timer."""
        self.timer.stop()
        self.indicator.set_status_active()

    def alarm(self):
        """Displays notifications and update the counter of the pomodoro."""
        # Update the UI (counter, active task, ...)
        if self.ui.selected_task == Window.TASK_POMODORO:
            self.ui.pomodoros_counter += 1
            n = self.settings.get_int('pomodoros-until-long-break')
            if not self.ui.pomodoros_counter % n:
                self.ui.selected_task = Window.TASK_LONG_BREAK
            else:
                self.ui.selected_task = Window.TASK_SHORT_BREAK
        else:
            self.ui.selected_task = Window.TASK_POMODORO

        self.ui.toggle_buttons()
        self.__update()

        # Show/play alarms
        if self.settings.get_boolean('osd-notification'):
            self._osd.display(
                self._ALARM_OSD_TITLE,
                Handler._ALARM_OSD_MSG)

        if self.settings.get_boolean('tone-notification'):
            self._sound.alarm()

        # Do this after updating the UI, otherwise the indicator might get
        # reset accidentally.
        self.indicator.alarm()

        self.ui.alarm()

    def __update(self):
        """Updates the interface.

        Private method to update the launcher and
        the window interface at once.
        """
        self.ui.update()
        self._launcher.update()
        self.indicator.update()

        if self.timer.status == Timer.STATUS_STOPPED:
            return False

        if self.timer.status == Timer.STATUS_COMPLETED:
            self.stop_timer()
            self.alarm()
            return False

        return True

    def _get_osd_messages(self, task):
        """Get the title and the message of the task.

        Used for notification.
        """
        task = '_' + task.replace('-', '_').upper()
        title = getattr(Handler, task + '_OSD_TITLE')
        message = getattr(Handler, task + '_OSD_MSG')
        return title, message

    def main(self):
        """Display the UI."""
        self.ui.show_all()
