### BEGIN LICENSE
# Copyright (C) 2012 <Elio Esteves Duarte> <elio.esteves.duarte@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


# Borrowed from Tomatoro create by Pierre Quillery.
# https://github.com/dandelionmood/Tomatoro
# Thanks Pierre!

from gi.repository import GObject


class Timer(object):
    """General-purpose timer class used to measure time.

    Attributes:
        STATUS_STOPPED: A string indicating that the timer loop is running.
        STATUS_RUNNING: A string indicating that the timer loop is nothing
                        running.
        STATUS_COMPLETED: A string indicating that the timer stopped because
                          the loop ended.
        status: It is used for other modules to check the current
                state of the timer, which can be STATUS_RUNNING,
                STATUS_STOPPED or STATUS_COMPLETED.
        seconds: A integer indicating the timer loop length in seconds.
        __tick_counter: Count the seconds after timer loop start.
    """
    STATUS_STOPPED = 'stopped'
    STATUS_RUNNING = 'running'
    STATUS_COMPLETED = 'completed'

    def __init__(self):
        """Initializes the Timer with the default values"""
        self.status = Timer.STATUS_STOPPED
        self.seconds = 0
        self.__tick_counter = 0

    def start(self, seconds):
        """Starts up the timer.

        Starts the timer loop with the given seconds.

        Args:
            seconds: Timer loop length in seconds.
        """
        self.status = Timer.STATUS_RUNNING
        self.seconds = seconds
        self.__tick_counter = 0
        GObject.timeout_add(1000, self.__tick)

    def stop(self):
        """Stops the timer.

        Changes the timer state to stopped and resets the __tick_counter.
        """
        self.status = Timer.STATUS_STOPPED
        self.__tick_counter = 0

    @property
    def time_mmss(self):
        """Retrieves the remaining time in format mm:ss.

        Returns:
            A string with the remaining time of timer in format mm:ss. For
            example: 24:59, 24:58.
        """
        seconds_left = self.seconds - self.__tick_counter
        minutes, seconds = divmod(seconds_left, 60)
        return "{0:0>2}:{1:0>2}".format(minutes, seconds)

    @property
    def time_ratio(self):
        """Retrieves the percentage of the remaining time.

        Returns:
            A number between 0.0 and 1.0. For example: 0.1, 0.9.
        """
        return ((self.__tick_counter * 100.0) / self.seconds) / 100.0

    def __tick(self):
        """Timer loop.

        Method executed every second to update the counter.
        """
        if self.status != Timer.STATUS_RUNNING:
            return False

        self.__tick_counter += 1

        # If the time is over change the status and brake de loop
        if self.__tick_counter > self.seconds:
            self.status = Timer.STATUS_COMPLETED
            return False

        return True
