### BEGIN LICENSE
# Copyright (C) 2012 <Elio Esteves Duarte> <elio.esteves.duarte@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import locale
from locale import gettext as _

from gi.repository import Gdk, Gio, Granite, Gtk
from tomate.helpers import get_settings

locale.textdomain('tomate')


class PreferencesDialog(Gtk.Dialog):

    def __init__(self, parent):
        Gtk.Dialog.__init__(self, _('Preferences'),
                            parent, Gtk.DialogFlags.MODAL,
                            (Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE),
                            type_hint=Gdk.WindowTypeHint.DIALOG,
                            resizable=False)
        self.set_size_request(370, 200)
        self.settings = get_settings()
        self.create_layout()

    def create_layout(self):
        """Main layout of the dialog."""
        notebook = Granite.WidgetsStaticNotebook.new()
        notebook.append_page(self._get_general_box(),
                             Gtk.Label(_('General')))
        notebook.append_page(self._get_notification_box(),
                             Gtk.Label(_('Notifications')))
        notebook.show_all()
        content_area = self.get_content_area()
        content_area.add(notebook)

    def _get_general_box(self):
        """Layout of general notebook page.

        Returns:
            A Grid gtk object.
        """
        grid = Gtk.Grid(row_spacing=6,
                        column_spacing=6,
                        margin_left=12,
                        margin_right=12,
                        margin_top=12,
                        margin_bottom=12)

        section_name = self._get_time_section_name()
        grid.attach(section_name, 0, 0, 1, 1)

        label, setting = self._get_pomodoro_length_setting()
        grid.attach(label, 0, 1, 1, 1)
        grid.attach_next_to(setting,
                            label,
                            Gtk.PositionType.RIGHT, 3, 1)

        label, setting = self._get_short_break_length_setting()
        grid.attach(label, 0, 2, 1, 1)
        grid.attach_next_to(setting,
                            label,
                            Gtk.PositionType.RIGHT, 3, 1)

        label, setting = self._get_long_break_length_setting()
        grid.attach(label, 0, 3, 1, 1)
        grid.attach_next_to(setting,
                            label,
                            Gtk.PositionType.RIGHT, 3, 1)

        section_name = self._get_behaviour_section_name()
        grid.attach(section_name, 0, 4, 1, 1)

        label, setting = self._get_show_progress_in_indicator_setting()
        grid.attach(label, 0, 5, 1, 1)
        grid.attach_next_to(setting,
                            label,
                            Gtk.PositionType.RIGHT, 3, 1)
        return grid

    def _get_time_section_name(self):
        """Label of the time length section.

        The name of the time section

        Returns:
            A Label gtk object.
        """
        section_name = Gtk.Label('<b>{0}</b>'
                                 .format(_('Timer Lengths:')),
                                 use_markup=True)
        section_name.set_halign(Gtk.Align.START)
        return section_name

    def _get_pomodoro_length_setting(self):
        """Changes the time length of pomodoro.

        Sets the length of the pomodoro in minutes.

        Returns:
            A tuple with a Label and SpinButton gtk objects.
        """
        label = Gtk.Label(_('Pomodoro:'),
                          margin_left=12)
        label.set_hexpand(True)
        label.set_halign(Gtk.Align.END)

        setting = Gtk.SpinButton.new_with_range(1, 99, 1)
        setting.set_hexpand(True)
        setting.set_halign(Gtk.Align.START)
        self.settings.bind('pomodoro-length',
                           setting,
                           'value',
                           Gio.SettingsBindFlags.DEFAULT)
        return label, setting

    def _get_short_break_length_setting(self):
        """Changes the time length of short break.

        Sets the length of the short break in minutes.

        Returns:
            A tuple with a Label and SpinButton gtk objects.
         """
        label = Gtk.Label(_('Short break:'),
                          margin_left=12)
        label.set_hexpand(True)
        label.set_halign(Gtk.Align.END)

        setting = Gtk.SpinButton.new_with_range(1, 99, 1)
        setting.set_hexpand(True)
        setting.set_halign(Gtk.Align.START)
        self.settings.bind('short-break-length',
                           setting,
                           'value',
                           Gio.SettingsBindFlags.DEFAULT)
        return label, setting

    def _get_long_break_length_setting(self):
        """Changes the time length of long break.

        Sets the length of the long break in minutes.

        Returns:
            A tuple with a Label and SpinButton gtk objects.
        """
        label = Gtk.Label(_('Long break:'),
                          margin_left=12)
        label.set_hexpand(True)
        label.set_halign(Gtk.Align.END)

        setting = Gtk.SpinButton.new_with_range(1, 99, 1)
        setting.set_hexpand(True)
        setting.set_halign(Gtk.Align.START)
        self.settings.bind('long-break-length',
                           setting,
                           'value',
                           Gio.SettingsBindFlags.DEFAULT)
        return label, setting

    def _get_behaviour_section_name(self):
        """Label of behaviour section.

        Name of the behaviour section.

        Returns:
            A Label gtk object.
        """
        section_name = Gtk.Label('<b>{0}</b>'
                                 .format(_('Behaviours:')),
                                 use_markup=True)
        section_name.set_halign(Gtk.Align.START)
        return section_name

    def _get_show_progress_in_indicator_setting(self):
        """Changes the show-progress-in-indicator setting.

        Configuration to show the timer progress in the indicator.

        Returns:
            A tuple with a Label and Switch gtk objects.
        """
        label = Gtk.Label(_('Show time progress in indicator:'),
                          margin_left=12)
        label.set_hexpand(True)
        label.set_halign(Gtk.Align.END)

        switch = Gtk.Switch()
        switch.set_hexpand(True)
        switch.set_halign(Gtk.Align.START)
        self.settings.bind('show-progress-in-indicator',
                           switch,
                           'active',
                           Gio.SettingsBindFlags.DEFAULT)
        return label, switch

    def _get_notification_box(self):
        """Layout of the notification notebook page.

        Returns:
            A Grid gtk object.
        """
        grid = Gtk.Grid(row_spacing=6,
                        column_spacing=6,
                        margin_left=12,
                        margin_right=12,
                        margin_top=12,
                        margin_bottom=6)

        label, setting = self._get_osd_notification_setting()
        grid.attach(label, 0, 0, 1, 1)
        grid.attach_next_to(setting,
                            label,
                            Gtk.PositionType.RIGHT, 3, 1)

        label, setting = self._get_audio_notification_setting()
        grid.attach(label, 0, 1, 1, 1)
        grid.attach_next_to(setting,
                            label,
                            Gtk.PositionType.RIGHT, 3, 1)
        return grid

    def _get_osd_notification_setting(self):
        """"Changes the osd-notification setting.

        Configuration to show a on screen display in the start and the end of
        a task.

        Returns:
            A tuple with a Label and Switch gtk objects."""
        label = Gtk.Label(_('Notify OSD:'),
                            margin_left=12)
        label.set_hexpand(True)
        label.set_halign(Gtk.Align.END)
        switch = Gtk.Switch()
        switch.set_hexpand(True)
        switch.set_halign(Gtk.Align.START)
        self.settings.bind('osd-notification',
                           switch,
                           'active',
                           Gio.SettingsBindFlags.DEFAULT)
        return label, switch

    def _get_audio_notification_setting(self):
        """Changes the audio notification setting.

        Configuration to ring alarm when the time ends.

        Returns:
            A tuple with a Label and Switch gtk objects.
        """
        label = Gtk.Label(_('Ring alarm when time ends:'))
        label.set_hexpand(True)
        label.set_halign(Gtk.Align.END)
        switch = Gtk.Switch()
        switch.set_hexpand(True)
        switch.set_halign(Gtk.Align.START)
        self.settings.bind('tone-notification',
                           switch,
                           'active',
                           Gio.SettingsBindFlags.DEFAULT)
        return label, switch
