# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 <Elio Esteves Duarte> <elio.esteves.duarte@gmail.com>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import locale
from locale import gettext as _

from gi.repository import GdkPixbuf, Granite, Gtk
from tomate.helpers import get_help_uri, get_media_file, show_uri
from tomate.timer import Timer
from tomate.helpers import get_settings
from tomate.widgets.about import AboutDialog
from tomate.widgets.preferences import PreferencesDialog

locale.textdomain('tomate')


class Window(Gtk.Window):
    """Application interface."""

    TASK_POMODORO = 'pomodoro'
    TASK_SHORT_BREAK = 'short-break'
    TASK_LONG_BREAK = 'long-break'
    TASK_INDEX = {TASK_POMODORO: 0,
                  TASK_SHORT_BREAK: 1,
                  TASK_LONG_BREAK: 2}
    TASK_NAME = dict((value, key) for key, value in TASK_INDEX.items())
    _ICON = GdkPixbuf.Pixbuf.new_from_file(get_media_file('tomate-small.png'))
    _MINUTE_TEMPLATE = '<span font="60">{0:0>2}:00</span>'
    _MINUTE_SECOND_TEMPLATE = '<span font="60">{0}</span>'
    _POMODOROS_TEMPLATE = '<span font="12">{0} pomodoros</span>'

    def __init__(self, handler):
        Gtk.Window.__init__(self,
                            title='Tomate',
                            icon=self._ICON,
                            window_position=Gtk.WindowPosition.CENTER,
                            resizable=False)
        self.set_size_request(380, 200)
        self.connect('delete-event', self.on_window_delete_event)

        self.handler = handler
        self.pomodoros_counter = 0
        self.settings = get_settings()
        self._create_layout()

    def _create_layout(self):
        """Main layout of the window."""
        container = Gtk.VBox()
        container.pack_start(self._get_toolbar(), False, False, 0)
        container.pack_start(self._get_elapsed_box(), True, True, 0)
        separator = Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL)
        container.pack_start(separator, False, False, 8)
        container.pack_start(self._get_modebutton(), True, True, 0)
        self.add(container)
        self.update()

    def _get_toolbar(self):
        """Returns the toolbar object."""
        # Toolbar
        toolbar = Gtk.Toolbar(toolbar_style=Gtk.ToolbarStyle.ICONS,
                              icon_size=Gtk.IconSize.LARGE_TOOLBAR,
                              orientation=Gtk.Orientation.HORIZONTAL)
        context = toolbar.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)

        # Start/stop button
        self.timer_button = Gtk.ToolButton(stock_id='gtk-media-play')
        self.timer_button.set_tooltip_text(_('Starts the timer'))
        self.timer_button.connect('clicked', self.on_timer_button_clicked)
        toolbar.insert(self.timer_button, -1)

        # Reset button
        self.reset_button = Gtk.ToolButton(stock_id='gtk-refresh')
        self.reset_button.set_tooltip_text(_('Resets the count of the pomodoros'))
        self.reset_button.connect('clicked', self.on_reset_clicked)
        self.reset_button.set_sensitive(False)
        toolbar.insert(self.reset_button, -1)

        # Space
        space = Gtk.SeparatorToolItem(draw=False)
        space.set_expand(True)
        toolbar.insert(space, -1)
        toolbar.insert(self._get_appmenu(), -1)
        return toolbar

    def _get_appmenu(self):
        """Returns the appmenu object."""
        menu = Gtk.Menu()

        # Settings Menu
        settings_mnu = Gtk.MenuItem.new_with_label(_('Preferences'))
        settings_mnu.connect('activate', self.on_settings_mnu_activate)
        menu.append(settings_mnu)

        # Help Menu
        help_mnu = Gtk.MenuItem.new_with_label(_('Help'))
        help_mnu.connect('activate', self.on_help_mnu_activate)
        menu.append(help_mnu)

        # About Menu
        about_mnu = Gtk.MenuItem.new_with_label(_('About'))
        about_mnu.connect('activate', self.on_about_mnu_activate)
        menu.append(about_mnu)

        # Appmenu
        icon = Gtk.Image.new_from_stock('gtk-properties',
                                        Gtk.IconSize.LARGE_TOOLBAR)
        appmenu = Granite.WidgetsToolButtonWithMenu.new(
            icon, 'Menu', menu,
            Granite.WidgetsToolButtonWithMenuMenuPosition.CENTER)
        return appmenu

    def _get_elapsed_box(self):
        """Returns the central frame."""
        # Frame border
        frame = Gtk.Frame(shadow_type=Gtk.ShadowType.IN,
                          margin_left=12,
                          margin_right=12,
                          margin_top=12,
                          margin_bottom=2)

        box = Gtk.VBox()

        # Elapsed Time
        self.elapsed_time = Gtk.Label(justify=Gtk.Justification.CENTER)
        self.elapsed_time.set_hexpand(True)
        self.elapsed_time.set_vexpand(True)
        self.elapsed_time.set_halign(Gtk.Align.FILL)
        self.elapsed_time.set_valign(Gtk.Align.FILL)
        box.pack_start(self.elapsed_time, True, True, 0)

        # Pomodoros Counter
        self.pomodoros_elapsed = Gtk.Label(justify=Gtk.Justification.CENTER,
                                           margin_bottom=12)
        self.pomodoros_elapsed.set_hexpand(True)
        box.pack_start(self.pomodoros_elapsed, False, False, 0)

        frame.add(box)
        return frame

    def _get_modebutton(self):
        """Returns the modebutton object."""
        self.modebutton = Granite.WidgetsModeButton.new()
        self.modebutton.set_margin_left(12)
        self.modebutton.set_margin_right(12)
        self.modebutton.set_margin_bottom(12)
        self.modebutton.append_text(_('Pomodoro'))
        self.modebutton.append_text(_('Short Break'))
        self.modebutton.append_text(_('Long Break'))
        self.modebutton.set_active(0)
        self.modebutton.connect('mode_changed',
                                self.on_modebutton_mode_changed)
        return self.modebutton

    def update(self):
        """Update the labels of the timer and the session."""
        if self.handler.timer.status == Timer.STATUS_RUNNING:
            time = self.handler.timer.time_mmss
            time_markup = Window._MINUTE_SECOND_TEMPLATE.format(time)
        else:
            time = self.settings.get_int(self.selected_task + '-length')
            time_markup = Window._MINUTE_TEMPLATE.format(time)

        self.elapsed_time.set_markup(time_markup)

        session_markup = Window._POMODOROS_TEMPLATE.format(
            self.pomodoros_counter)
        self.pomodoros_elapsed.set_markup(session_markup)

    def toggle_buttons(self):
        """Enables or disables the buttons."""
        if self.handler.timer.status == Timer.STATUS_RUNNING:
            self.timer_button.set_property('stock_id', 'gtk-media-stop')
            self.timer_button.set_tooltip_text(_('Stops the timer'))
            self.modebutton.set_sensitive(False)
            self.reset_button.set_sensitive(False)
        else:
            self.timer_button.set_property('stock_id', 'gtk-media-play')
            self.timer_button.set_tooltip_text(_('Starts the timer'))
            self.modebutton.set_sensitive(True)
            if self.pomodoros_counter != 0:
                self.reset_button.set_sensitive(True)

    @property
    def selected_task(self):
        """ Get the current task mode. """
        return Window.TASK_NAME[self.modebutton.get_selected()]

    @selected_task.setter
    def selected_task(self, task):
        """ Set the current task mode. """
        self.modebutton.set_selected(Window.TASK_INDEX[task])

    def alarm(self):
        self.show()
        self.present()

    ############################################################################
    # EVENT HANDLERS
    ############################################################################

    def on_timer_button_clicked(self, widget, data=None):
        """Starts or stops the timer through the handler."""
        if self.handler.timer.status == Timer.STATUS_RUNNING:
            self.handler.stop_timer()
        else:
            self.handler.start_timer()

        self.toggle_buttons()
        self.update()

    def on_reset_clicked(self, widget, data=None):
        """Resets the counter of the pomodoros."""
        self.pomodoros_counter = 0
        self.reset_button.set_sensitive(False)
        self.update()

    def on_modebutton_mode_changed(self, button, data=None):
        """Changes the current task."""
        self.handler.indicator.set_status_active()
        self.update()

    def on_settings_mnu_activate(self, widget, data=None):
        """Opens the preferences dialog."""
        dialog = PreferencesDialog(self)
        dialog.run()
        dialog.destroy()
        self.update()
        self.handler.indicator.update()

    def on_help_mnu_activate(self, widget, data=None):
        """ Opens the content window """
        show_uri(self, "ghelp:%s" % get_help_uri())

    def on_about_mnu_activate(self, widget, data=None):
        """Opens the about dialog."""
        dialog = AboutDialog(self)
        dialog.about.run()
        dialog.about.destroy()

    def on_window_delete_event(self, widget, data=None):
        """Close the window to the notification area or quit"""
        if self.handler.timer.status == Timer.STATUS_RUNNING:
            self.handler.indicator.show_mnu.set_label(_('Show'))
            return self.hide_on_delete()
        Gtk.main_quit()
